package smartresource.core;

import java.util.*;
import jade.core.Agent;  	

import jade.core.behaviours.SimpleBehaviour;


public abstract class ReusableAtomicBehavior extends SimpleBehaviour implements ParametrizedAsset{

	public boolean finished=true; //by default, is a OneShotBehavior
	public boolean success=true; //by default, ends in success

	Vector startParameters;

	public void setStartParameters(Vector startParams){
		startParameters=startParams;
	}

	public int onEnd(){
		Vector addBeliefs;
		Vector removeBeliefs;
		if(success==true){
			addBeliefs=getParameterValues("addOnSuccess",SmartResourceAgent.APL_NS);
			removeBeliefs=getParameterValues("removeOnSuccess",SmartResourceAgent.APL_NS);
		}
		else{
			addBeliefs=getParameterValues("addOnFail",SmartResourceAgent.APL_NS);
			removeBeliefs=getParameterValues("removeOnFail",SmartResourceAgent.APL_NS);
		}

		addBeliefs.addAll(getParameterValues("addOnEnd",SmartResourceAgent.APL_NS));
		removeBeliefs.addAll(getParameterValues("removeOnEnd",SmartResourceAgent.APL_NS));

		for(int i=0; i<removeBeliefs.size();i++)
			removeBelief(new SemanticStatement((String)removeBeliefs.elementAt(i)));

		for(int i=0; i<addBeliefs.size();i++)
			addBelief(new SemanticStatement((String)addBeliefs.elementAt(i)));

		wakeAgent();
		return 0;
	}

	public boolean done(){
		return finished;
	}


// Interface to subclasses, i.e. RABs

	public String getParameterValue(String name){
		return getParameterValue(name, null);
	}

	public String getParameterValue(String name, String ns){
	        if(startParameters!=null){
			Iterator it=startParameters.iterator(); 
			while(it.hasNext()){ 
				BehaviorStartParameter p=(BehaviorStartParameter)it.next();
				if(name.equals(p.name))
					if(ns==null || ns.equals(p.name_ns))
						return p.value;
			}
		}
		return null;
	}

	public Vector getParameterValues(String name){
		return getParameterValues(name, null);
	}

	public Vector getParameterValues(String name, String ns){
	        Vector values=new Vector();
	        if(startParameters!=null){
			Iterator it=startParameters.iterator(); 
			while(it.hasNext()){ 
				BehaviorStartParameter p=(BehaviorStartParameter)it.next();
				if(name.equals(p.name)) 
					if(ns==null || ns.equals(p.name_ns))
						values.add(p.value);
			}
		}
		return values;
	}

	public void print(String text){
		((SmartResourceAgent)myAgent).print(text);
	}

	public void wakeAgent(){
		((SmartResourceAgent)myAgent).restartLive();
	}

	public void addBelief(SemanticStatement belief){
		((SmartResourceAgent)myAgent).beliefs.add(belief);		
	}

	public void removeBelief(SemanticStatement belief){
	        boolean found;
	        do{
			found=((SmartResourceAgent)myAgent).beliefs.remove(belief);		
		}while(found);
	}

	public boolean hasBelief(SemanticStatement belief){
		return ((SmartResourceAgent)myAgent).beliefs.contains(belief);
	}

	public Vector getBeliefs(SemanticStatement template){
		int ind=((SmartResourceAgent)myAgent).beliefs.indexOf(template);
		if(ind==-1) return null;
		else{
			Vector result=new Vector();
			result.add(((SmartResourceAgent)myAgent).beliefs.elementAt(ind));
			ind=((SmartResourceAgent)myAgent).beliefs.indexOf(template, ind+1);
			while(ind!=-1){
				result.add(((SmartResourceAgent)myAgent).beliefs.elementAt(ind));
				ind=((SmartResourceAgent)myAgent).beliefs.indexOf(template, ind+1);
			}
		 	return result;
		}
	 }

}
