package smartresource.core;

import java.util.HashMap;
import java.util.Vector;
import java.util.StringTokenizer;

public class SemanticStatement{

	public String subject=null;
	public String predicate=null;	
	public String object=null;

	private void fix(){
		if(subject==null) subject="?";
		if(predicate==null) predicate="?";
		if(object==null) object="?";

		subject=subject.trim();
		predicate=predicate.trim();
		object=object.trim();
	}
	
	public SemanticStatement(String s, String p, String o){
		subject=s;
		predicate=p;
		object=o;

		fix();
	}

	public SemanticStatement(String str){
		if(str!=null){
			StringTokenizer token=new StringTokenizer(str," ");
			subject=token.hasMoreTokens()?token.nextToken():"?";
			predicate=token.hasMoreTokens()?token.nextToken():"?";
			object=token.hasMoreTokens()?token.nextToken():"?";
		}
		fix();
	}

	public SemanticStatement(Object toCopy){
		if(toCopy instanceof SemanticStatement){
			subject=((SemanticStatement)toCopy).subject;
			predicate=((SemanticStatement)toCopy).predicate;
			object=((SemanticStatement)toCopy).object;
		}		
	}

	//returns true if subject and predicate and object are all not '?', i.e known
	public boolean isFullyDefined(){
		if(object.equals("?") || predicate.equals("?") || subject.equals("?"))
			return false;
		else return true;
	} 

	public boolean isSubjectUndefined(){
		return subject.equals("?");
	}
	public boolean isPredicateUndefined(){
		return predicate.equals("?");
	}
	public boolean isObjectUndefined(){
		return object.equals("?");
	}

	public boolean isSubjectAny(){
		return subject.startsWith("*");
	}
	public boolean isPredicateAny(){
		return predicate.startsWith("*");
	}
	public boolean isObjectAny(){
		return object.startsWith("*");
	}

	public boolean hasVariables(){
		if(subject.startsWith("*") && subject.length()>2 && subject.endsWith("*")) return true;
		if(predicate.startsWith("*") && predicate.length()>2 && predicate.endsWith("*")) return true;
		if(object.startsWith("*") && object.length()>2 && object.endsWith("*")) return true;
		return false;
	}

	public String getSubjectVariableName(){
		if(subject.startsWith("*") && subject.length()>2 && subject.endsWith("*"))
			return subject.substring(1,subject.length()-1);
		else return null;
	}
	public String getPredicateVariableName(){
		if(predicate.startsWith("*") && predicate.length()>2 && predicate.endsWith("*"))
			return predicate.substring(1,predicate.length()-1);
		else return null;
	}
	public String getObjectVariableName(){
		if(object.startsWith("*") && object.length()>2 && object.endsWith("*"))
			return object.substring(1,object.length()-1);
		else return null;
	}


	//returns true this statement is equal (taking universalities into account) to the 'obj' statement
	public boolean equals (Object obj){
		SemanticStatement other=(SemanticStatement)obj;
		if(!subject.startsWith("*") && !other.subject.startsWith("*")) 
			if(!subject.equals(other.subject)) return false;
		if(!predicate.startsWith("*") && !other.predicate.startsWith("*")) 
			if(!predicate.equals(other.predicate)) return false;
		if(!object.startsWith("*") && !other.object.startsWith("*")) 
			if(!object.equals(other.object)) return false;

		return true;
	}

	//replaces variables with their values from vars
	public boolean bindVars(HashMap vars){
		boolean changed=false;
		if(vars.size()>0){
			String var=getSubjectVariableName();
			if(var!=null){
				String value=(String)vars.get(var);
				if(value!=null && !value.equals("")){ subject=value; changed=true;}
			}
			var=getPredicateVariableName();
			if(var!=null){
				String value=(String)vars.get(var);
				if(value!=null && !value.equals("")) { predicate=value; changed=true;}
			}
			var=getObjectVariableName();
			if(var!=null){
				String value=(String)vars.get(var);
				if(value!=null && !value.equals("")) { object=value; changed=true;}
			}
		}
		return changed; 
	}

	public String toString(){
		return subject+" "+predicate+" "+object;
	}
}
