package smartresource.shared;

import smartresource.core.*;

import java.io.*;
import java.net.*;

public class HttpDataFetcherBehavior extends ReusableAtomicBehavior{

	String uri;
	String saveTo;
	
	Thread fetchIt;

	boolean tried=false;

	public void action() {

	        if(fetchIt!=null){ // control is in the fetching thread
        		block();
        		return; 
        	}

   		if(tried && finished) return;

		finished=false;
		tried=true;

		uri=getParameterValue("url"); 
		uri.equals(""); //to cause exception if null
		saveTo=getParameterValue("saveTo"); 
		saveTo.equals(""); //to cause exception if null

		Runnable runIt = new FetchDocument();
		fetchIt = new Thread(runIt);
		fetchIt.start();

		block();
	}

	private String fetch(String aUrlName){
		final String fNEWLINE = System.getProperty("line.separator");
		URL aURL=null;
		try {
			aURL=new URL(aUrlName);
		}
	        catch ( MalformedURLException ex ) {
		      ex.printStackTrace(); return null;
		}

		boolean ok=true;
		StringBuffer result = new StringBuffer();
		BufferedReader reader = null;
		try {
		      reader = new BufferedReader( new InputStreamReader(aURL.openStream()) );
		      String line = null;
		      while ( (line = reader.readLine()) != null) {
		          result.append(line);
		          result.append(fNEWLINE);
		      }
		}
	        catch (IOException ex) {
		      print("Failed to retrieve " + aURL);
		      ok=false;
		}
		finally{
		        try {
			     if(reader != null) reader.close();
			}
			catch (IOException ex){
			      ex.printStackTrace(); return null;
			}
		}

		if(ok) return result.toString();
		else return null;
	}

	private class FetchDocument implements Runnable{
		public void run() {
			try{
				String str=fetch(uri);
				if(str==null){
					success=false;
					finished=true;
					fetchIt=null;				
					restart();
					return;
				}
				File datafile=new File(saveTo);
				FileWriter fwrite;
				fwrite=new FileWriter(datafile, false);
				BufferedWriter ff=new BufferedWriter(fwrite);
				ff.write(str,0,str.length());
				ff.close();

				finished=true;
				fetchIt=null;				
				restart();

			}catch(Exception e){
				e.printStackTrace();
				finished=true;
				success=false;
				fetchIt=null;				
				restart();
			}
		}
	}

}
